# Guardian Code
import board, time, neopixel, digitalio, pwmio, adafruit_vl53l1x
import mount_sd
from adafruit_motor import servo
from audiopwmio import PWMAudioOut as AudioOut
from audiocore import WaveFile

# set up distance sensor
i2c = board.STEMMA_I2C()
distance_sensor = adafruit_vl53l1x.VL53L1X(i2c)
distance_sensor

# set up servo
pwm = pwmio.PWMOut(board.GP16, frequency=50)
# Change min_pulse / max_pulse values to calibrate servo arm in either direction.
# min ~ 500, max ~ 2500. Defaults are below & can be eliminated.
servo_1 = servo.Servo(pwm, min_pulse = 750, max_pulse = 2250)

# set up the speaker
from audiomp3 import MP3Decoder
from audiopwmio import PWMAudioOut as AudioOut
audio = AudioOut(board.GP15)
path = "/sd/guardian/"

filename = "guardian-activate.mp3"
mp3_file = open(path + filename, "rb")
decoder = MP3Decoder(mp3_file)

# play_sound function - pass in the FULL NAME of file to play
def play_mp3(filename):
    decoder.file = open(path + filename, "rb")
    audio.play(decoder)
    while audio.playing:
        pass

light_pin = board.GP0
light_num = 20
light = neopixel.NeoPixel(light_pin, light_num)
RED = (255, 0, 0)
BLUE = (0, 0, 255)
BLACK = (0, 0, 0)

def sweep(curr_angle, sweep_time):
    for degree in range(curr_angle,181, 5):
        servo_1.angle = degree
        time.sleep(sweep_time)
    for degree in range(180, -1, -5):
        servo_1.angle = degree
        time.sleep(sweep_time)
    for degree in range(0, curr_angle+1, 5):
        servo_1.angle = degree
        time.sleep(sweep_time)

def sweep_pulse(curr_angle, sweep_time):
    global positive
    light.fill(BLACK)
    lighter = True
    degree = curr_angle
    brightness = 0.05
    light.brightness = brightness
    light.fill(BLUE)
    while brightness > 0.0:
        if degree > 175:
            positive = False
        if degree < 5:
            positive = True
        if positive:
            degree += 5
            servo_1.angle = degree
            time.sleep(sweep_time)
        else:
            degree -= 5
            servo_1.angle = degree
            time.sleep(sweep_time)

        if lighter:
            brightness+=0.05
            print(brightness)
        else:
            brightness-=0.05
        light.brightness = brightness
        if brightness >= 1.0:
            lighter = False
    return positive



def pulse(color, pulse_time):
    light.brightness = 0.0
    light.fill(color)
    for idx in range(0, 101, 5):
        light.brightness = idx/100.0
        time.sleep(pulse_time)
    for idx in range(100,-1,-5):
        light.brightness = idx/100.0
        time.sleep(pulse_time/2)

def flicker(flicker_time):
    light.brightness = 0.0
    light.fill(BLUE)
    light[19] = RED
    for idx in range(0, 60):
        light.brightness = idx/100.0
        time.sleep(flicker_time)
        if idx>11 and idx<50:
            light.brightness = (idx-10)/100.0
        time.sleep(flicker_time)
    for idx in range(60, 101):
        light.brightness = idx/100.0
        time.sleep(flicker_time/5)
    time.sleep(1.0)

DIST_ALERT = 10
DIST_ACTIVE = 1
alert = False
active = False
alert_counter = 0
positive = True

distance_sensor.start_ranging()
servo_1.angle = 90
light.fill(BLACK)

while True:
    if distance_sensor.data_ready:
        distance = distance_sensor.distance
        if distance == None:
            distance = 31
        if distance < DIST_ACTIVE:
            if not active:
                servo_1.angle = 90
                time.sleep(0.05)
                pulse(RED, 0.02)
                play_mp3(filename)
            active = True
            alert = False
            alert_counter = 0
        elif distance < DIST_ALERT or active:
            alert = True
            active = False
            alert_counter = 0
        else:
            active = False
        if alert_counter > 4:
            alert = False
        print((distance,))
    if not alert and not active:
        light.fill(BLACK)
    if active:
        flicker(0.1)
        light.fill(BLUE)
        sweep(90, 0.02)
        pulse(BLUE, 0.05)
        pulse(BLUE, 0.05)
        pulse(BLUE, 0.05)
    if alert:
        angle = servo_1.angle
        print(angle)
        positive = sweep_pulse(angle, 0.07)
        alert_counter+=1
